@@ -1,20 +1,20 @@
-/*
- *  Copyright © 2017-2025 Wellington Wallace
+/**
+ * Copyright © 2017-2025 Wellington Wallace
  *
- *  This file is part of Easy Effects.
+ * This file is part of Easy Effects.
  *
- *  Easy Effects is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * Easy Effects is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  Easy Effects is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
+ * Easy Effects is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with Easy Effects. If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Easy Effects. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "fir_filter_base.hpp"
@@ -22,10 +22,13 @@
 #include <sys/types.h>
 #include <zita-convolver.h>
 #include <algorithm>
+#include <chrono>
 #include <cmath>
 #include <cstddef>
+#include <format>
 #include <numbers>
 #include <string>
+#include <thread>
 #include <utility>
 #include <vector>
 #include "util.hpp"
@@ -43,12 +46,20 @@ FirFilterBase::FirFilterBase(std::string tag) : log_tag(std::move(tag)) {}
 FirFilterBase::~FirFilterBase() {
   zita_ready = false;
 
+  free_zita();
+}
+
+void FirFilterBase::free_zita() {
   if (conv != nullptr) {
     conv->stop_process();
 
-    conv->cleanup();
+    while (!conv->check_stop()) {
+      std::this_thread::sleep_for(std::chrono::milliseconds(1));
+    }
 
     delete conv;
+
+    conv = nullptr;
   }
 }
 
@@ -82,18 +93,17 @@ auto FirFilterBase::create_lowpass_kernel(const float& cutoff, const float& tran
     return output;
   }
 
-  /*
-    transition band frequency as a fraction of the sample rate
-  */
+  // transition band frequency as a fraction of the sample rate
 
   const float b = transition_band / static_cast<float>(rate);
 
-  /*
-      The kernel size must be odd: M + 1 where M is even. This is done so it can be symmetric around the main lobe
-      https://www.dspguide.com/ch16/1.htm
-
-      The kernel size is related to the transition bandwidth M = 4/BW
-  */
+  /**
+   * The kernel size must be odd: M + 1 where M is even. This is done so it can
+   * be symmetric around the main lobe
+   * https://www.dspguide.com/ch16/1.htm
+   *
+   * The kernel size is related to the transition bandwidth M = 4/BW
+   */
 
   auto M = static_cast<uint>(std::ceil(4.0F / b));
 
@@ -101,18 +111,14 @@ auto FirFilterBase::create_lowpass_kernel(const float& cutoff, const float& tran
 
   output.resize(M + 1U);
 
-  /*
-    cutoff frequency as a fraction of the sample rate
-  */
+  // cutoff frequency as a fraction of the sample rate
 
   const float fc = cutoff / static_cast<float>(rate);
 
   float sum = 0.0F;
 
   for (size_t n = 0U; n < output.size(); n++) {
-    /*
-      windowed-sinc kernel https://www.dspguide.com/ch16/1.htm
-    */
+    // windowed-sinc kernel https://www.dspguide.com/ch16/1.htm
 
     if (n == M / 2U) {
       output[n] = 2.0F * std::numbers::pi_v<float> * fc;
@@ -121,22 +127,18 @@ auto FirFilterBase::create_lowpass_kernel(const float& cutoff, const float& tran
                   static_cast<float>(n - static_cast<uint>(M / 2U));
     }
 
-    /*
-      Blackman window https://www.dspguide.com/ch16/1.htm
-    */
+    // Blackman window https://www.dspguide.com/ch16/1.htm
 
-    const float w = 0.42F -
-                    0.5F * std::cos(2.0F * std::numbers::pi_v<float> * static_cast<float>(n) / static_cast<float>(M)) +
-                    0.08F * std::cos(4.0F * std::numbers::pi_v<float> * static_cast<float>(n) / static_cast<float>(M));
+    const float w =
+        0.42F - (0.5F * std::cos(2.0F * std::numbers::pi_v<float> * static_cast<float>(n) / static_cast<float>(M))) +
+        (0.08F * std::cos(4.0F * std::numbers::pi_v<float> * static_cast<float>(n) / static_cast<float>(M)));
 
     output[n] *= w;
 
     sum += output[n];
   }
 
-  /*
-    Normalizing so that we have unit gain at zero frequency
-  */
+  // Normalizing so that we have unit gain at zero frequency
 
   std::ranges::for_each(output, [&](auto& v) { v /= sum; });
 
@@ -150,22 +152,16 @@ void FirFilterBase::setup_zita() {
     return;
   }
 
-  if (conv != nullptr) {
-    conv->stop_process();
-
-    conv->cleanup();
-
-    delete conv;
-  }
+  free_zita();
 
   conv = new Convproc();
 
   conv->set_options(0);
 
-  int ret = conv->configure(2, 2, kernel.size(), n_samples, n_samples, n_samples, 0.0F /*density*/);
+  int ret = conv->configure(2, 2, kernel.size(), n_samples, n_samples, n_samples, 0.0F /* density */);
 
   if (ret != 0) {
-    util::warning(log_tag + "can't initialise zita-convolver engine: " + util::to_string(ret, ""));
+    util::warning(std::format("{}can't initialise zita-convolver engine: {}", log_tag, ret));
 
     return;
   }
@@ -173,7 +169,7 @@ void FirFilterBase::setup_zita() {
   ret = conv->impdata_create(0, 0, 1, kernel.data(), 0, static_cast<int>(kernel.size()));
 
   if (ret != 0) {
-    util::warning(log_tag + "left impdata_create failed: " + util::to_string(ret, ""));
+    util::warning(std::format("{}left impdata_create failed: {}", log_tag, ret));
 
     return;
   }
@@ -181,7 +177,7 @@ void FirFilterBase::setup_zita() {
   ret = conv->impdata_create(1, 1, 1, kernel.data(), 0, static_cast<int>(kernel.size()));
 
   if (ret != 0) {
-    util::warning(log_tag + "right impdata_create failed: " + util::to_string(ret, ""));
+    util::warning(std::format("{}right impdata_create failed: {}", log_tag, ret));
 
     return;
   }
@@ -189,7 +185,7 @@ void FirFilterBase::setup_zita() {
   ret = conv->start_process(CONVPROC_SCHEDULER_PRIORITY, CONVPROC_SCHEDULER_CLASS);
 
   if (ret != 0) {
-    util::warning(log_tag + "start_process failed: " + util::to_string(ret, ""));
+    util::warning(std::format("{}start_process failed: {}", log_tag, ret));
 
     conv->stop_process();
     conv->cleanup();
