@@ -1,30 +1,32 @@
-/*
- *  Copyright © 2017-2024 Wellington Wallace
+/**
+ * Copyright © 2017-2025 Wellington Wallace
  *
- *  This file is part of Easy Effects.
+ * This file is part of Easy Effects.
  *
- *  Easy Effects is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * Easy Effects is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  Easy Effects is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
+ * Easy Effects is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with Easy Effects. If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Easy Effects. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "ladspa_wrapper.hpp"
 #include <dlfcn.h>
 #include <ladspa.h>
 #include <sys/types.h>
+#include <algorithm>
 #include <cassert>
 #include <cmath>
 #include <cstdlib>
 #include <cstring>
+#include <format>
 #include <limits>
 #include <span>
 #include <string>
@@ -39,7 +41,7 @@ static inline const char* get_ladspa_path() {
   const char* path = std::getenv("LADSPA_PATH");
 
   if (path == nullptr || path[0] == '\0') {
-    path = "/usr/local/lib/ladspa/:" LIB_DIR "/ladspa/:/usr/lib/ladspa/";
+    path = "/usr/local/lib/ladspa/:" LIB_DIR "/ladspa/:/usr/lib/ladspa/:/usr/lib64/ladspa/";
   }
 
   return path;
@@ -292,19 +294,19 @@ static inline LADSPA_Data get_port_default_raw(const LADSPA_Descriptor* descript
         return max;
       } else if (LADSPA_IS_HINT_LOGARITHMIC(hint->HintDescriptor)) {
         if (LADSPA_IS_HINT_DEFAULT_LOW(hint->HintDescriptor)) {
-          return std::exp(std::log(min) * 0.75 + std::log(max) * 0.25);
+          return std::exp((std::log(min) * 0.75) + (std::log(max) * 0.25));
         } else if (LADSPA_IS_HINT_DEFAULT_MIDDLE(hint->HintDescriptor)) {
-          return std::exp(std::log(min) * 0.5 + std::log(max) * 0.5);
+          return std::exp((std::log(min) * 0.5) + (std::log(max) * 0.5));
         } else if (LADSPA_IS_HINT_DEFAULT_HIGH(hint->HintDescriptor)) {
-          return std::exp(std::log(min) * 0.25 + std::log(max) * 0.75);
+          return std::exp((std::log(min) * 0.25) + (std::log(max) * 0.75));
         }
       } else {
         if (LADSPA_IS_HINT_DEFAULT_LOW(hint->HintDescriptor)) {
-          return min * 0.75F + max * 0.25F;
+          return (min * 0.75F) + (max * 0.25F);
         } else if (LADSPA_IS_HINT_DEFAULT_MIDDLE(hint->HintDescriptor)) {
-          return min * 0.5F + max * 0.5F;
+          return (min * 0.5F) + (max * 0.5F);
         } else if (LADSPA_IS_HINT_DEFAULT_HIGH(hint->HintDescriptor)) {
-          return min * 0.25F + max * 0.75F;
+          return (min * 0.25F) + (max * 0.75F);
         }
       }
     }
@@ -322,13 +324,9 @@ static inline LADSPA_Data clamp_port_value(const LADSPA_Descriptor* descriptor,
 
   get_port_bounds(descriptor, port, rate, min, max);
 
-  if (val < min) {
-    val = min;
-  }
+  val = std::max(val, min);
 
-  if (val > max) {
-    val = max;
-  }
+  val = std::min(val, max);
 
   return val;
 }
@@ -606,7 +604,7 @@ void LadspaWrapper::connect_data_ports(const std::span<const float>& left_in,
   }
 
   if (right_in_idx != null_ul) {
-    descriptor->connect_port(instance, right_in_idx, const_cast<LADSPA_Data*>(left_in.data()));
+    descriptor->connect_port(instance, right_in_idx, const_cast<LADSPA_Data*>(right_in.data()));
   }
 
   if (probe_left_idx == null_ul || probe_right_idx == null_ul) {
@@ -658,7 +656,7 @@ void LadspaWrapper::connect_data_ports(const std::span<const float>& left_in,
   }
 
   if (right_out_idx != null_ul) {
-    descriptor->connect_port(instance, right_out_idx, left_out.data());
+    descriptor->connect_port(instance, right_out_idx, right_out.data());
   }
 }
 
@@ -765,7 +763,7 @@ auto LadspaWrapper::get_control_port_value(const std::string& symbol) const -> f
   auto iter = map_cp_name_to_idx.find(symbol);
 
   if (iter == map_cp_name_to_idx.end()) {
-    util::warning(plugin_name + " port symbol not found: " + symbol);
+    util::warning(std::format("{} port symbol not found: {}", plugin_name, symbol));
 
     return 0.0F;
   }
@@ -791,7 +789,7 @@ auto LadspaWrapper::set_control_port_value_clamp(const std::string& symbol, floa
   auto iter = map_cp_name_to_idx.find(symbol);
 
   if (iter == map_cp_name_to_idx.end()) {
-    util::warning(plugin_name + " port symbol not found: " + symbol);
+    util::warning(std::format("{} port symbol not found: {}", plugin_name, symbol));
 
     return value;
   }
