@@ -1,33 +1,33 @@
-/*
- *  Copyright © 2017-2024 Wellington Wallace
+/**
+ * Copyright © 2017-2025 Wellington Wallace
  *
- *  This file is part of Easy Effects.
+ * This file is part of Easy Effects.
  *
- *  Easy Effects is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
+ * Easy Effects is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  Easy Effects is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
+ * Easy Effects is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with Easy Effects. If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Easy Effects. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "plugin_base.hpp"
-#include <gio/gio.h>
-#include <glib-object.h>
-#include <glib.h>
-#include <glib/gi18n.h>
 #include <pipewire/filter.h>
 #include <pipewire/keys.h>
 #include <pipewire/loop.h>
 #include <pipewire/port.h>
 #include <pipewire/properties.h>
 #include <pipewire/thread-loop.h>
+#include <qnamespace.h>
+#include <qobjectdefs.h>
+#include <qthread.h>
+#include <qtimer.h>
 #include <spa/node/io.h>
 #include <spa/param/latency-utils.h>
 #include <spa/param/latency.h>
@@ -38,16 +38,22 @@
 #include <spa/utils/defs.h>
 #include <spa/utils/hook.h>
 #include <sys/types.h>
+#include <KLocalizedString>
+#include <QString>
 #include <algorithm>
 #include <array>
 #include <chrono>
+#include <cmath>
 #include <cstddef>
 #include <cstdint>
+#include <format>
 #include <span>
 #include <string>
 #include <thread>
 #include <utility>
-#include "pipe_manager.hpp"
+#include "db_manager.hpp"
+#include "pipeline_type.hpp"
+#include "pw_manager.hpp"
 #include "tags_app.hpp"
 #include "tags_plugin_name.hpp"
 #include "util.hpp"
@@ -64,28 +70,33 @@ void on_process(void* userdata, spa_io_position* position) {
     return;
   }
 
+  // We had to add the following checks for dummy array sizes. See #4085
+  if (d->pb->dummy_left.size() != n_samples) {
+    d->pb->dummy_left.resize(n_samples);
+
+    std::ranges::fill(d->pb->dummy_left, 0.0F);
+  }
+
+  if (d->pb->dummy_right.size() != n_samples) {
+    d->pb->dummy_right.resize(n_samples);
+
+    std::ranges::fill(d->pb->dummy_right, 0.0F);
+  }
+
   if (rate != d->pb->rate || n_samples != d->pb->n_samples) {
     d->pb->rate = rate;
     d->pb->n_samples = n_samples;
 
-    d->pb->dummy_left.resize(n_samples);
-    d->pb->dummy_right.resize(n_samples);
-
-    std::ranges::fill(d->pb->dummy_left, 0.0F);
-    std::ranges::fill(d->pb->dummy_right, 0.0F);
-
-    d->pb->clock_start = std::chrono::system_clock::now();
+    d->pb->got_null_left_in = false;
+    d->pb->got_null_left_out = false;
+    d->pb->got_null_right_in = false;
+    d->pb->got_null_right_out = false;
+    d->pb->got_null_probe = false;
 
     d->pb->setup();
   }
 
-  d->pb->delta_t = 0.001F * static_cast<float>(std::chrono::duration_cast<std::chrono::milliseconds>(
-                                                   std::chrono::system_clock::now() - d->pb->clock_start)
-                                                   .count());
-
-  d->pb->send_notifications = d->pb->delta_t >= d->pb->notification_time_window;
-
-  // util::warning("processing: " + util::to_string(n_samples));
+  // util::warning("Processing: " + util::to_string(n_samples));
 
   auto* in_left = static_cast<float*>(pw_filter_get_dsp_buffer(d->in_left, n_samples));
   auto* in_right = static_cast<float*>(pw_filter_get_dsp_buffer(d->in_right, n_samples));
@@ -101,24 +112,48 @@ void on_process(void* userdata, spa_io_position* position) {
   if (in_left != nullptr) {
     left_in = std::span(in_left, n_samples);
   } else {
+    if (!d->pb->got_null_left_in) {
+      util::warning("Processing: we received a null left_in pointer. Using the dummy array instead.");
+
+      d->pb->got_null_left_in = true;
+    }
+
     left_in = d->pb->dummy_left;
   }
 
   if (in_right != nullptr) {
     right_in = std::span(in_right, n_samples);
   } else {
+    if (!d->pb->got_null_right_in) {
+      util::warning("Processing: we received a null right_in pointer. Using the dummy array instead.");
+
+      d->pb->got_null_right_in = true;
+    }
+
     right_in = d->pb->dummy_right;
   }
 
   if (out_left != nullptr) {
     left_out = std::span(out_left, n_samples);
   } else {
+    if (!d->pb->got_null_left_out) {
+      util::warning("Processing: we received a null left_out pointer. Using the dummy array instead.");
+
+      d->pb->got_null_left_out = true;
+    }
+
     left_out = d->pb->dummy_left;
   }
 
   if (out_right != nullptr) {
     right_out = std::span(out_right, n_samples);
   } else {
+    if (!d->pb->got_null_right_out) {
+      util::warning("Processing: we received a null right_out pointer. Using the dummy array instead.");
+
+      d->pb->got_null_right_out = true;
+    }
+
     right_out = d->pb->dummy_right;
   }
 
@@ -129,6 +164,12 @@ void on_process(void* userdata, spa_io_position* position) {
     auto* probe_right = static_cast<float*>(pw_filter_get_dsp_buffer(d->probe_right, n_samples));
 
     if (probe_left == nullptr || probe_right == nullptr) {
+      if (!d->pb->got_null_probe) {
+        util::debug("Processing: we received a null pointer for probe left/right. Using the dummy array instead.");
+
+        d->pb->got_null_probe = true;
+      }
+
       std::span l(d->pb->dummy_left.data(), n_samples);
       std::span r(d->pb->dummy_right.data(), n_samples);
 
@@ -140,16 +181,14 @@ void on_process(void* userdata, spa_io_position* position) {
       d->pb->process(left_in, right_in, left_out, right_out, l, r);
     }
   }
-
-  if (d->pb->send_notifications) {
-    d->pb->clock_start = std::chrono::system_clock::now();
-
-    d->pb->send_notifications = false;
-  }
 }
 
-auto update_filter(struct spa_loop* loop, bool async, uint32_t seq, const void* data, size_t size, void* user_data)
-    -> int {
+auto update_filter([[maybe_unused]] struct spa_loop* loop,
+                   [[maybe_unused]] bool async,
+                   [[maybe_unused]] uint32_t seq,
+                   [[maybe_unused]] const void* data,
+                   [[maybe_unused]] size_t size,
+                   void* user_data) -> int {
   auto* self = static_cast<PluginBase*>(user_data);
 
   spa_process_latency_info latency_info{};
@@ -169,7 +208,10 @@ auto update_filter(struct spa_loop* loop, bool async, uint32_t seq, const void*
   return 0;
 }
 
-void on_filter_state_changed(void* userdata, pw_filter_state old, pw_filter_state state, const char* error) {
+void on_filter_state_changed(void* userdata,
+                             [[maybe_unused]] pw_filter_state old,
+                             pw_filter_state state,
+                             [[maybe_unused]] const char* error) {
   auto* d = static_cast<PluginBase::data*>(userdata);
 
   d->pb->state = state;
@@ -195,49 +237,58 @@ void on_filter_state_changed(void* userdata, pw_filter_state old, pw_filter_stat
   }
 }
 
-const struct pw_filter_events filter_events = {.state_changed = on_filter_state_changed, .process = on_process};
+const struct pw_filter_events filter_events = {.version = 0,
+                                               .destroy = nullptr,
+                                               .state_changed = on_filter_state_changed,
+                                               .io_changed = nullptr,
+                                               .param_changed = nullptr,
+                                               .add_buffer = nullptr,
+                                               .remove_buffer = nullptr,
+                                               .process = on_process,
+                                               .drained = nullptr,
+                                               .command = nullptr};
 
 }  // namespace
 
 PluginBase::PluginBase(std::string tag,
-                       std::string plugin_name,
-                       std::string package,
-                       const std::string& schema,
-                       const std::string& schema_path,
-                       PipeManager* pipe_manager,
+                       QString plugin_name,
+                       QString package,
+                       QString instance_id,
+                       pw::Manager* pipe_manager,
                        PipelineType pipe_type,
                        const bool& enable_probe)
     : log_tag(std::move(tag)),
       name(std::move(plugin_name)),
       package(std::move(package)),
+      instance_id(std::move(instance_id)),
       pipeline_type(pipe_type),
       enable_probe(enable_probe),
-      settings(g_settings_new_with_path(schema.c_str(), schema_path.c_str())),
-      global_settings(g_settings_new(tags::app::id)),
-      pm(pipe_manager) {
-  std::string description;
+      pm(pipe_manager),
+      baseWorker(new PluginBaseWorker),
+      native_ui_timer(new QTimer(this)) {
+  QString description;
+  QString description_pipeline;
+
+  switch (pipeline_type) {
+    case PipelineType::input:
+      description_pipeline = i18n("(Mic)");
+      break;
+    case PipelineType::output:
+      description_pipeline = i18n("(Speakers)");
+      break;
+  }
 
   if (name != "output_level" && name != "spectrum") {
-    description = tags::plugin_name::get_translated()[name];
-
-    bypass = g_settings_get_boolean(settings, "bypass") != 0;
-
-    gconnections.push_back(g_signal_connect(settings, "changed::bypass",
-                                            G_CALLBACK(+[](GSettings* settings, char* key, gpointer user_data) {
-                                              auto* self = static_cast<PluginBase*>(user_data);
-
-                                              self->bypass = g_settings_get_boolean(settings, "bypass") != 0;
-                                            }),
-                                            this));
+    description = tags::plugin_name::Model::self().translate(name) + " " + description_pipeline;
   } else if (name == "output_level") {
-    description = _("Output Level Meter");
+    description = i18n("Output Level Meter");
   } else if (name == "spectrum") {
-    description = _("Spectrum");
+    description = i18n("Spectrum");
   }
 
   pf_data.pb = this;
 
-  const auto filter_name = "ee_" + log_tag.substr(0U, log_tag.size() - 2U) + "_" + name;
+  const auto filter_name = "ee_" + log_tag.substr(0U, log_tag.size() - 2U) + "_" + name.toStdString();
 
   pm->lock();
 
@@ -245,8 +296,8 @@ PluginBase::PluginBase(std::string tag,
 
   pw_properties_set(props_filter, PW_KEY_APP_ID, tags::app::id);
   pw_properties_set(props_filter, PW_KEY_NODE_NAME, filter_name.c_str());
-  pw_properties_set(props_filter, PW_KEY_NODE_NICK, name.c_str());
-  pw_properties_set(props_filter, PW_KEY_NODE_DESCRIPTION, description.c_str());
+  pw_properties_set(props_filter, PW_KEY_NODE_NICK, name.toStdString().c_str());
+  pw_properties_set(props_filter, PW_KEY_NODE_DESCRIPTION, description.toStdString().c_str());
   pw_properties_set(props_filter, PW_KEY_MEDIA_TYPE, "Audio");
   pw_properties_set(props_filter, PW_KEY_MEDIA_CATEGORY, "Filter");
   pw_properties_set(props_filter, PW_KEY_MEDIA_ROLE, "DSP");
@@ -325,11 +376,28 @@ PluginBase::PluginBase(std::string tag,
   }
 
   pm->sync_wait_unlock();
+
+  native_ui_timer->setInterval(static_cast<long>(1000.0 / DbMain::lv2uiUpdateFrequency()));
+
+  connect(native_ui_timer, &QTimer::timeout, this, [&]() {
+    if (lv2_wrapper == nullptr || !lv2_wrapper->has_ui()) {
+      return;
+    }
+
+    lv2_wrapper->notify_ui();
+    lv2_wrapper->update_ui();
+  });
+
+  // worker thread for the native ui and maybe also other things
+
+  baseWorker->moveToThread(&workerThread);
+
+  workerThread.start();
+
+  connect(&workerThread, &QThread::finished, baseWorker, &QObject::deleteLater);
 }
 
 PluginBase::~PluginBase() {
-  post_messages = false;
-
   pm->lock();
 
   if (listener.link.next != nullptr || listener.link.prev != nullptr) {
@@ -340,22 +408,11 @@ PluginBase::~PluginBase() {
 
   pm->sync_wait_unlock();
 
-  for (auto& handler_id : gconnections) {
-    g_signal_handler_disconnect(settings, handler_id);
-  }
-
-  gconnections.clear();
-
-  g_object_unref(settings);
+  workerThread.quit();
+  workerThread.wait();
 }
 
-void PluginBase::set_post_messages(const bool& state) {
-  post_messages = state;
-}
-
-void PluginBase::reset_settings() {
-  util::reset_all_keys_except(settings);
-}
+void PluginBase::reset() {}
 
 auto PluginBase::connect_to_pw() -> bool {
   connected_to_pw = false;
@@ -367,12 +424,12 @@ auto PluginBase::connect_to_pw() -> bool {
   if (pw_filter_connect(filter, PW_FILTER_FLAG_RT_PROCESS, nullptr, 0) != 0) {
     pm->unlock();
 
-    util::warning(log_tag + name + " cannot connect the filter to PipeWire!");
+    util::warning(std::format("{}{} cannot connect the filter to PipeWire!", log_tag, name.toStdString()));
 
     return false;
   }
 
-  initialize_listener();
+  pw_filter_add_listener(filter, &listener, &filter_events, &pf_data);
 
   pm->sync_wait_unlock();
 
@@ -380,7 +437,7 @@ auto PluginBase::connect_to_pw() -> bool {
     std::this_thread::sleep_for(std::chrono::milliseconds(1));
 
     if (state == PW_FILTER_STATE_ERROR) {
-      util::warning(log_tag + name + " is in an error");
+      util::warning(std::format("{}{} is in an error", log_tag, name.toStdString()));
 
       return false;
     }
@@ -392,10 +449,11 @@ auto PluginBase::connect_to_pw() -> bool {
 
   pm->sync_wait_unlock();
 
-  /*
-    The filter we link in our pipeline have at least 4 ports. Some have six. Before we try to link filters we have to
-    wait until the information about their ports is available in PipeManager's list_ports vector.
-  */
+  /**
+   * The filter we link in our pipeline have at least 4 ports. Some have six.
+   * Before we try to link filters we have to wait until the information about
+   * their ports is available in PipeManager's list_ports vector.
+   */
 
   while (pm->count_node_ports(node_id) != n_ports) {
     std::this_thread::sleep_for(std::chrono::milliseconds(1));
@@ -403,15 +461,11 @@ auto PluginBase::connect_to_pw() -> bool {
 
   connected_to_pw = true;
 
-  util::debug(log_tag + name + " successfully connected to PipeWire graph");
+  util::debug(std::format("{}{} successfully connected to PipeWire graph", log_tag, name.toStdString()));
 
   return true;
 }
 
-void PluginBase::initialize_listener() {
-  pw_filter_add_listener(filter, &listener, &filter_events, &pf_data);
-}
-
 auto PluginBase::get_node_id() const -> uint {
   return node_id;
 }
@@ -437,43 +491,66 @@ void PluginBase::disconnect_from_pw() {
   pm->sync_wait_unlock();
 
   node_id = SPA_ID_INVALID;
+
+  while (pw_filter_get_state(filter, nullptr) != PW_FILTER_STATE_UNCONNECTED) {
+    std::this_thread::sleep_for(std::chrono::milliseconds(1));
+  }
+
+  util::warning(std::format("{}{} is disconnected", log_tag, name.toStdString()));
 }
 
 void PluginBase::setup() {}
 
-void PluginBase::process(std::span<float>& left_in,
-                         std::span<float>& right_in,
-                         std::span<float>& left_out,
-                         std::span<float>& right_out) {}
+void PluginBase::process([[maybe_unused]] std::span<float>& left_in,
+                         [[maybe_unused]] std::span<float>& right_in,
+                         [[maybe_unused]] std::span<float>& left_out,
+                         [[maybe_unused]] std::span<float>& right_out) {}
 
-void PluginBase::process(std::span<float>& left_in,
-                         std::span<float>& right_in,
-                         std::span<float>& left_out,
-                         std::span<float>& right_out,
-                         std::span<float>& probe_left,
-                         std::span<float>& probe_right) {}
+void PluginBase::process([[maybe_unused]] std::span<float>& left_in,
+                         [[maybe_unused]] std::span<float>& right_in,
+                         [[maybe_unused]] std::span<float>& left_out,
+                         [[maybe_unused]] std::span<float>& right_out,
+                         [[maybe_unused]] std::span<float>& probe_left,
+                         [[maybe_unused]] std::span<float>& probe_right) {}
 
 auto PluginBase::get_latency_seconds() -> float {
   return 0.0F;
 }
 
-void PluginBase::show_native_ui() {
-  if (lv2_wrapper == nullptr) {
-    return;
-  }
+void PluginBase::showNativeUi() {
+  native_ui_timer->start();
 
-  if (!lv2_wrapper->has_ui()) {
-    lv2_wrapper->load_ui();
-  }
+  // using invokeMethod to force this code to run in the base class worker thread and
+  // avoid load in QML or main thread.
+
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        if (lv2_wrapper != nullptr && !lv2_wrapper->has_ui()) {
+          lv2_wrapper->load_ui();
+        }
+      },
+      Qt::QueuedConnection);
+
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
-void PluginBase::close_native_ui() {
-  if (lv2_wrapper == nullptr) {
-    return;
-  }
+void PluginBase::closeNativeUi() {
+  native_ui_timer->stop();
 
-  lv2_wrapper->native_ui_to_gsettings();
-  lv2_wrapper->close_ui();
+  // NOLINTBEGIN(clang-analyzer-cplusplus.NewDeleteLeaks)
+  QMetaObject::invokeMethod(
+      baseWorker,
+      [this] {
+        if (lv2_wrapper != nullptr) {
+          lv2_wrapper->native_ui_to_database();
+          lv2_wrapper->close_ui();
+        }
+      },
+      Qt::QueuedConnection);
+
+  // NOLINTEND(clang-analyzer-cplusplus.NewDeleteLeaks)
 }
 
 void PluginBase::set_native_ui_update_frequency(const uint& value) {
@@ -481,81 +558,67 @@ void PluginBase::set_native_ui_update_frequency(const uint& value) {
     return;
   }
 
-  lv2_wrapper->set_ui_update_rate(value);
+  native_ui_timer->setInterval(static_cast<long>(1000.0 / value));
 }
 
 void PluginBase::get_peaks(const std::span<float>& left_in,
                            const std::span<float>& right_in,
                            std::span<float>& left_out,
                            std::span<float>& right_out) {
-  if (!post_messages) {
-    return;
+  float in_left_max = 0.0F;
+  float in_right_max = 0.0F;
+  float out_left_max = 0.0F;
+  float out_right_max = 0.0F;
+
+  for (size_t i = 0; i < n_samples; ++i) {
+    in_left_max = std::max(in_left_max, std::fabs(left_in[i]));
+    in_right_max = std::max(in_right_max, std::fabs(right_in[i]));
+    out_left_max = std::max(out_left_max, std::fabs(left_out[i]));
+    out_right_max = std::max(out_right_max, std::fabs(right_out[i]));
   }
 
-  // input level
-
-  float peak_l = std::ranges::max(left_in);
-  float peak_r = std::ranges::max(right_in);
-
-  input_peak_left = (peak_l > input_peak_left) ? peak_l : input_peak_left;
-  input_peak_right = (peak_r > input_peak_right) ? peak_r : input_peak_right;
-
-  // output level
-
-  peak_l = std::ranges::max(left_out);
-  peak_r = std::ranges::max(right_out);
-
-  output_peak_left = (peak_l > output_peak_left) ? peak_l : output_peak_left;
-  output_peak_right = (peak_r > output_peak_right) ? peak_r : output_peak_right;
+  input_peak_left = util::linear_to_db(in_left_max);
+  input_peak_right = util::linear_to_db(in_right_max);
+  output_peak_left = util::linear_to_db(out_left_max);
+  output_peak_right = util::linear_to_db(out_right_max);
 }
 
-void PluginBase::setup_input_output_gain() {
-  input_gain = static_cast<float>(util::db_to_linear(g_settings_get_double(settings, "input-gain")));
-  output_gain = static_cast<float>(util::db_to_linear(g_settings_get_double(settings, "output-gain")));
-
-  g_signal_connect(settings, "changed::input-gain", G_CALLBACK(+[](GSettings* settings, char* key, gpointer user_data) {
-                     auto* self = static_cast<PluginBase*>(user_data);
-
-                     self->input_gain = util::db_to_linear(g_settings_get_double(settings, key));
-                   }),
-                   this);
-
-  g_signal_connect(settings, "changed::output-gain",
-                   G_CALLBACK(+[](GSettings* settings, char* key, gpointer user_data) {
-                     auto* self = static_cast<PluginBase*>(user_data);
-
-                     self->output_gain = util::db_to_linear(g_settings_get_double(settings, key));
-                   }),
-                   this);
-}
-
-void PluginBase::apply_gain(std::span<float>& left, std::span<float>& right, const float& gain) {
+void PluginBase::apply_gain(std::span<float>& left, std::span<float>& right, const float& gain) const {
   if (left.empty() || right.empty()) {
     return;
   }
 
-  std::ranges::for_each(left, [&](auto& v) { v *= gain; });
-  std::ranges::for_each(right, [&](auto& v) { v *= gain; });
-}
+  float* __restrict l_ptr = left.data();
+  float* __restrict r_ptr = right.data();
 
-void PluginBase::notify() {
-  const auto input_peak_db_l = util::linear_to_db(input_peak_left);
-  const auto input_peak_db_r = util::linear_to_db(input_peak_right);
-
-  const auto output_peak_db_l = util::linear_to_db(output_peak_left);
-  const auto output_peak_db_r = util::linear_to_db(output_peak_right);
-
-  input_level.emit(input_peak_db_l, input_peak_db_r);
-  output_level.emit(output_peak_db_l, output_peak_db_r);
-
-  input_peak_left = util::minimum_linear_level;
-  input_peak_right = util::minimum_linear_level;
-  output_peak_left = util::minimum_linear_level;
-  output_peak_right = util::minimum_linear_level;
+  for (uint i = 0; i < n_samples; i++) {
+    l_ptr[i] *= gain;
+    r_ptr[i] *= gain;
+  }
 }
 
 void PluginBase::update_probe_links() {}
 
 void PluginBase::update_filter_params() {
-  pw_loop_invoke(pw_thread_loop_get_loop(pm->thread_loop), update_filter, 1, nullptr, 0, false, this);
+  pw_loop_invoke(pw_thread_loop_get_loop(pm->thread_loop), update_filter, 1, nullptr, 0, false, this);  // NOLINT
+}
+
+float PluginBase::getInputLevelLeft() const {
+  return input_peak_left;
+}
+
+float PluginBase::getInputLevelRight() const {
+  return input_peak_right;
+}
+
+float PluginBase::getOutputLevelLeft() const {
+  return output_peak_left;
+}
+
+float PluginBase::getOutputLevelRight() const {
+  return output_peak_right;
+}
+
+bool PluginBase::hasNativeUi() {
+  return lv2_wrapper->has_ui();
 }
